#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from configuration import CONFIG
from gestionOMS import *
from gestion_unites import *
from gestion_donnees import calcule_max_graphique, convertit_jours_vers_python

import matplotlib.pyplot as plt

# Essentiellement, la fonction qui trace la courbe


def cree_figure(conf,l_jours,l_poids,typedonnee,liste_err):
    debug("debut de cree_figure",liste_err)
    try:
        liste_data_labels_p,liste_data_labels_z = renvoie_liste_labels(conf,CONFIG["liste_data_choisie_p"],CONFIG["liste_data_choisie_z"],liste_err)
    except:
        erreur("bug avec liste data labels",liste_err)
        return ""

    # maxi saisi par l'utilisateur ?
    if conf["maxi"] ==0:
        # Est-ce qu'on a donné un maxi quand même (car même échelle) ?
        print(conf)
        print(conf["non_sauve"])
        if conf["non_sauve"].get("maxi",0) == 0:
            jour_maxi = calcule_max_graphique(l_jours)
        else:
            jour_maxi = conf["non_sauve"]["maxi"]+1
    else:
        jour_maxi = conf["maxi"]+1
        

    # On s'assure que c'est bien compris dans les bornes
    jour_maxi = max(CONFIG["jours_mini_courbe"],min(jour_maxi,CONFIG["jours_maxi_courbe"]))
    debug("cree_figure : gestion du jour max ok : "+str(jour_maxi),liste_err)    
    
    # si l'unité n'est pas précisée, ni en "non sauvé" ni par l'utilisateur
    if conf["unite"] == "" and conf["non_sauve"].get("unite","") == "":
        unite = choix_unite(jour_maxi)
        debug("Unité non précisée, on choisit "+unite,liste_err)
    elif conf["unite"] != "":
        unite = conf["unite"]
    else:
        unite = conf["non_sauve"]["unite"]
    
    #debug("prématurité dans la config : "+conf["prematurite"],liste_err)
    prema = int(convertit_jours_vers_python(conf["prematurite"],liste_err))
    ## Gestion des prémas, deux cas :
    # Si agecorrige est oui, alors on veut juste soustraire la valeur de préma
    # à toutes les données.
    # Si agecorrige est non, alors on veut ajouter la valeur de préma aux courbes de référence.

    debug("Prématurité : "+str(prema)+" age corrigé : "+conf["agecorrige"],liste_err)
            
    if prema>0 and conf["agecorrige"] == "oui":
        l_jours = [j-prema for j in l_jours]
        jour_maxi = jour_maxi - prema
#        debug("liste des jours : "+str(l_jours),liste_err)

    l_jours_conv = convertit_tableau(l_jours,unite,liste_err)
    # Attention, comme les jours commencent à partir de 0, faut enlever 1 pour avoir la borne...
    age_maxi = convertitunite(jour_maxi-1,unite,liste_err)
    
    debug("cree_figure : conversion des unités ok : "+str(l_jours_conv),liste_err)        

    titre = "Courbe de "+typedonnee+" OMS"
    
    if conf["typecourbe"] == "P":
        # percentiles
        liste_data_labels = liste_data_labels_p
        if conf["sexe"] == "M":
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["perc_garcon"]#f_poids_perc_garcon
            titre  += " (percentiles, garçon)"
        elif conf["sexe"] == "F":
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["perc_fille"]
            titre += " (percentiles, fille)"
        else:
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["perc_mixte"]
            titre += " (percentiles)"
    elif conf["typecourbe"] == "Z":
        liste_data_labels = liste_data_labels_z
        if conf["sexe"] == "M":
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["z_garcon"]
            titre += " (moyenne et écarts-types, garçon)"
        elif conf["sexe"] == "F":
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["z_fille"]
            titre += " (moyenne et écarts-types, fille)"
        else:
            fichier_oms = CONFIG["fichiersOMS"][typedonnee]["z_mixte"]
            titre += " (moyenne et écarts-types)"
    else:
        erreur("Type de courbe invalide"+conf["typecourbe"],liste_err)
        return ""
    
    # Si y'a un nom on met "courbe de machin"
    if conf["nom"] !="":
        titre += " de " +conf["nom"]

    
    if prema>0:
        titre+= ", préma de "+conf["prematurite"]
        if conf["agecorrige"] == "oui":
            titre+=" (courbe en âge corrigé)"
        else:
            titre+=" (courbe en âge réel, données OMS décalées)"
   
    debug("cree_figure : géré le type de courbe ok. Liste des data labels : "+str(liste_data_labels),liste_err)
    debug("Fichier d'où extraire les données : "+fichier_oms,liste_err)
    
        
    
    # convertir les unités
    #### Partie code OMS
    #### On extrait les données des courbes, et on convertit les jours dans l'unité voulues
    try:
        t = lire_fichier_csv(fichier_oms)
    except:
        liste_err[0].append("Impossible d'ouvrir le fichier "+fichier_oms)
        return ""
    
    debug("cree_figure : on va convertir données OMS à la bonne unité",liste_err)
    try:
        coljour= (extraire_colonne(t,0,jour_maxi))
        if prema>0 and conf["agecorrige"] != "oui":
            coljour = [j + prema for j in coljour]
        coljour = convertit_tableau(coljour,unite,liste_err)
    except:
        erreur("Problème à la conversion du tableau OMS. jour_maxi = "+str(jour_maxi)+" unite = "+unite,liste_err)
        return ""
    
    debug("cree_figure : prête à créer la figure",liste_err)   
    #### La figure, params
    
    fig = plt.figure(num=None, figsize=(conf["largeur"], conf["hauteur"]), dpi=100, facecolor=conf["couleurs"]["fond"])
    plt.rcParams['axes.facecolor'] = conf["couleurs"]["fond"]
    plt.rcParams['axes.edgecolor']= conf["couleurs"]["cadretxt"]
    plt.rcParams['xtick.color'] = conf["couleurs"]["cadretxt"]
    plt.rcParams['ytick.color'] = conf["couleurs"]["cadretxt"]
    plt.rcParams['grid.color'] = conf["couleurs"]["grille"]
    plt.rcParams['legend.edgecolor'] = conf["couleurs"]["grille"]

    #Tracé des courbes OMS
    for (i,label,couleur) in liste_data_labels:
        plt.plot(coljour,extraire_colonne(t,i,jour_maxi),label=label,color=couleur)
        #warning("colonne"+str(i)+str(label)+str(couleur),liste_err)

    debug("cree_figure : tracé des courbes OMS ok",liste_err) 
    
    
    ##### Tracé des courbes OMS fini
    
    # On extrait la valeur min et la valeur max des poids
    (colonne_min,_,_) = liste_data_labels[-1]
    (colonne_max,_,_) = liste_data_labels[0]
    
    debug("cree_figure : colonnes min et max : "+str(colonne_min)+" "+str(colonne_max),liste_err)
    
    poids_min = min(extraire_colonne(t,colonne_min,jour_maxi))
    poids_max = max(extraire_colonne(t,colonne_max,jour_maxi))
    if l_jours != []:
        poids_min = min(min(l_poids),poids_min)
        poids_max = max(max(l_poids),poids_max)
    # On ajuste un peu ces min et max
    # min : valeur min -1kg
    poids_min = max(0,poids_min-1)
    #max : +5% 
    poids_max = poids_max * 1.05
        
    
    debug("cree_figure : prête à tracer la courbe de l'enfant",liste_err)

    ### Tracé pour de bon    
    if l_jours != []:
        plt.plot(l_jours_conv,l_poids,label=conf["nom"],color=conf["couleurs"]["cadretxt"],marker='o')
        debug("Tracé de la courbe enfant, avec les jours "+str(l_jours_conv),liste_err)
        
        #### extrapolatios éventuelles
        #print("prolongercourbes" in conf)
        if conf["prolongercourbes"] == "oui":
            #print("coucou")
            # les dates, on prend tous les jours tant qu'à faire
            date_fin = int(l_jours[-1])
            dates_extrapole = list(range(date_fin, jour_maxi))
            donnees_extrapole =  prolongecourbe(t, date_fin, l_poids[-1], dates_extrapole, conf["typecourbe"], liste_err)
            #print("données extrapolées !")
            dates_extrapole = convertit_tableau(dates_extrapole,unite,liste_err)
            
            # tracé des données extrapolées
            plt.plot(dates_extrapole, donnees_extrapole,color=conf["couleurs"]["cadretxt"], linestyle=(0, (5,7)), marker=None)
        
    else:
        debug("On ne trace pas de courbe enfant", liste_err)

    plt.xlabel("Âge en "+unite,color=conf["couleurs"]["cadretxt"])
    plt.ylabel(typedonnee.capitalize()+" en "+CONFIG["unites_typedonnees"][typedonnee],color=conf["couleurs"]["cadretxt"])
    #print("bla")
    plt.title(titre,color=conf["couleurs"]["cadretxt"])
    #print("ble")
    if l_jours_conv == []:
        plt.axis([0,age_maxi, poids_min, poids_max])
    else:
        plt.axis([min(0,l_jours_conv[0]),age_maxi,poids_min,poids_max])
    #print("bli")

    
    if conf['legende']=="oui":
        legende = plt.legend(loc=conf['positionlegende'])
        plt.setp(legende.get_texts(), color=conf["couleurs"]["cadretxt"])
    plt.grid(conf["grille"]=="oui")

    fig.tight_layout()
    
    debug("Fin de cree_figure", liste_err)
    return fig



def prolongecourbe(tableauOMS, date, donnee, nouvdates, typecourbe, liste_err):
    """ tableauOMS est le ableau des données OMS. date et donnee sont la date (jours)
    et la dernière donnée d'où on extrapole. On veut les extrapolations à nouvdate 
    (tableaux de jours)
    typecourbe est P ou Z. Pour P il faut commencer à regarder à l'indice 4, pour Z
    à l'indice 1
    On renvoie [] si pas pu extrapoler. """
    
    # la ligne OMS
    ligneoms = tableauOMS[date]
    debug("prolongecourbe : Ligne OMS :"+str(ligneoms)+" valeur de donnée : "+str(donnee), liste_err)
    
    
    # On cherche dans quel intervalle on se situe
    if typecourbe == "P":
        idep=4
    else:
        idep = 1
    i = idep
    while i<len(ligneoms) and ligneoms[i]<donnee:
        i+=1
    debug("prolongecourbe : on a trouvé la valeur de i : "+str(i),liste_err)
    if i>=len(ligneoms):
        warning("prolongation de courbe : pas réussi... donnée au dessus !", liste_err)
        return []
    if i==idep:
        warning("prolongation de courbe : pas réussi... donnée en dessous !", liste_err)
        return []
    
    # Le coeff : donnee = coeff * l[i-1]+ (1-coeff)*ligne[i]
    # On utilisera la même chose pour la nouvelle donnee
    #print(ligneoms[i], ligneoms[i-1])
    coeff = (donnee - ligneoms[i])/(ligneoms[i-1] - ligneoms[i])
    #print("coeff : "+str(coeff))
    
    # extrapolations
    nouvdonnees = []
    for j in nouvdates:
        ligne2 = tableauOMS[j]
        nouvdonnees.append(coeff*ligne2[i-1]+ (1-coeff)*ligne2[i])

    return nouvdonnees